/* This program solves a set of coupled differtial equations descibing the structure of a NS: dp/dr=(G*M/r^2)*rho and dM/dr=4*PI*r^2*rho. The differential equations are represented by the array dydx[n] where n is the number of different equations. Here y[0] is the pressure and y[1] is the mass. The intial conditions are r=0, M(0)=0 and p(0)=po.The procedure stops when the pressure has reached a certain value pstop. The method of solving the equations is a fifth order Runge Kutta method (standard for this kind of problems), using Kash-Carp parameters. See "Numerical Recipes for C" chapter 16 for more details on this method. A simple variable stepsize is used according to Baym, Petchick and Sutherland 1971.*/
/* Programma is aangepast voor het oplossing van de structuur van een quarkster. Dit gebreurt voor een massareeks, waarna de massa's worden weggeschreven zodat een massa-straaldiagram kan worden gemaakt.
Voor het oplossen van mu uit de druk wordt gebruik gemaakt van een simpel numeriek oplosschema uit Nemerical Recipes voor C. De toenstantvergelijkingen zijn verkregen met behulp van eerste orde QCD-storingsrekening, moor het MS-barschema (hoewl eerste orde schema-onafhankelijk is). De labda_MS-schaal is bepaald volgens de methode van Fraga et al (hep-ph/0101143v1). Hiermee is alleen labda nog vrij.*/

#include <stdio.h>
#include <math.h>

#define NPOINTS 999
#define stdhe 197.32 /* hbar c naar MeV fm^-1 */
#include "qcdeqn_mas.h"
#include "settings.h"

extern stor vars;
extern double lschaal;
const double toNAT=1./(stdhe*stdhe*stdhe);

/* Het hoofdprogramma */
int main ()
{
int nmustep=NPOINTS,i;
double mu,mustep,mustart=0.,mumax=1000.0;
double pres[3],eps[3];


 
/*Open file to store data in. The results are written to an asci file*/
FILE *filenorm,*fileabs,*filealpha;

	filenorm=fopen("eosdata/eosnorm.asc","w");
	fileabs=fopen("eosdata/eosabs.asc","w");
	filealpha=fopen("eosdata/runningalpha.asc","w");

fprintf(filenorm,"# \t\tlschaal=2\n");
fprintf(filenorm,"# i\tMu\tP/P0\teps/eps0\n");
fprintf(fileabs,"# \t\tinter=0\t\tlschaal=2\n");
fprintf(fileabs,"# i\tMu\tP\teps\tP\teps\n");
fprintf(filealpha,"# Bendaring labda=1");
fprintf(filealpha,"# i\tmu\talpha\tdalpha\n");

mustep=(mumax-mustart)/nmustep;

for (i=1;i<nmustep+1;i++)
	{
		mu=mustart+(double)i*mustep;

		/*interactie aan, lschaal=2 */
		scaling=2.0;
		calc(mu);
		pres[2]=vars.p;
		eps[2]=vars.eps;
		if(pres[2]<0.||eps[2]<0.) {
			pres[2]=0.;
			eps[2]=0.;
		}

		/*Interactie uit*/
		pres[0]=-3.*vars.thf;
		eps[0]=3.*vars.thf - mu*3.*(dthpotfree(mu))*toNAT;

	
		fprintf(filenorm,"%i\t%g\t%g\t%g\n",i,mu,pres[2]/pres[0],eps[2]/eps[0]);
		fprintf(fileabs,"%i\t%g\t%g\t%g\t%g\t%g\n",i,mu,pres[0],eps[0],pres[2],eps[2]);
		fprintf(filealpha,"%i\t%g\t%g\t%g\t%g\n",i,mu,alpha(mu),dalpha(mu),lschaal);
	}
fclose(filenorm);
fclose(fileabs);
fclose(filealpha);

return 0;
}
