#include <stdio.h>
#include <math.h>
#include "natunits.h"
#include "qcdeqn_mas.h"
#include "solving.h"

#define lms 379.246 /* MeV volgens Fraga, Romatschke, PhysRevD 71, 105014 */
#define mhat 10e-10 /* 262 /* MeV volgens Fraga, Romatschke, PhysRevD 71, 105014 */
#define Nf 3.0 /* Model voor drie smaken*/
#define Nc 3.0 /* Model voor Nc=3 kleuren */

#define stdhe 197.32 /* hbar c naar MeV fm^-1 */
#define fm 1e-15 /* Een femtometer */

extern const double toNAT;

double B0=(11.0 - 2.0*Nf/3.0),B1=(51.0 - 19.0*Nf/3.0);
/*double B2=(2857.0 - 5033.0*Nf/9.0 + 325.0*Nf*Nf/27.0); Alleen voor 2e orde*/
double lambda,l,dl,ddl,alp,dalp,ddalp,ln,dln,ddln,u,du,ddu,m,dm,ddm,lschaal=2.;

stor vars;

/*Definieer alp,dalp,ln,dln,u,du,m,dm voor een bepaalde mu*/
void defin(double mu)

{
	lambda=lschaal*mu;
	l=2.0*log(lambda/lms);
	dl=2.0/(lambda);
	ddl=-2.0/(lambda*lambda);
	alp=alpha();
	dalp=dalpha();
	ddalp = ddalpha();
	m=massa(mu);
	dm=dmassa(mu);
	ddm=ddmassa(mu);
	u = sqrt(mu*mu-m*m);
	du = (2.*mu - 2.*m*dm)/(2.*sqrt(mu*mu - m*m));
	ddu = (-pow(m - mu*dm,2.) + m*(-pow(mu,2.) + pow(m,2.))*ddm)/pow(pow(mu,2.) - pow(m,2.),1.5);
	ln = log((mu + u)/m);
	dln = (m*(-(((mu + u)*dm)/(m*m)) + (1. + du)/m))/(mu + u);
	ddln = (pow(mu + u,2.)*pow(dm,2.) - m*pow(mu + u,2.)*ddm + pow(m,2.)*(-pow(1. + du,2.) + (mu + u)*ddu))/(pow(m,2.)*pow(mu + u,2.));
}

/* Functie voor running coupling */
double alpha ()
{
	double first,antw;
	first = -2.0*((B1/(B0*B0))*(log(l)/l));
	return((4.0*PI)/(B0*l)*(1.0+first));
}

double dalpha ()
{
	/*  dalpha/dmu = dalpha/dl  * dl/dlabda(=dl) * dlabda/dmu (=lschaal) */
	double antw;
	antw = -(8.*PI*(2.*B1 + B0*B0*l - 4.*B1*log(l)))/(pow(B0,3.)*pow(l,3.));
	return antw*dl*lschaal;
}
/* Tweeede afgeleide */
double ddalpha ()
{
	double antw;
	antw = (-4.*PI*(-2.*(pow(B0,2.)*l + B1*(5. - 6.*log(l)))*pow(dl,2.) + l*(2.*B1 + pow(B0,2.)*l - 4.*B1*log(l))*ddl))/(pow(B0,3.)*pow(l,4.));
	return antw*dl*lschaal;
}

/* Running massa */
double massa(double mu)
{
	return mhat*pow((alp/PI),4.0/9.0)*(1.0+0.895062*(alp/PI));
}
/* afgeleide */
double dmassa(double mu)
{
	double antw;
	antw = (mhat*(0.267216 + 0.247428*alp)*dalp)/(pow(alp,5./9.));
	return antw;
}
/* tweede afgeleide */

double ddmassa(double mu)
{
	double antw;
	antw = (mhat*((-0.14845317070298242 + 0.10996793568453099*alp)*pow(dalp,2.) + 0.24742785529019476*alp*(1.0799742290614995 + alp)*ddalp))/pow(alp,1.5555555555555556);
	return antw;
}

/* Thermodynamische potentiaal massa strange, 1e orde */
double thpotmas(double mu)
{
	double OM0,OM1;
	OM0 = -(Nc*((3./2.)*pow(m,4.)*ln + mu*(pow(mu,2.) - (5.*pow(m,2.))/2.)*u))/(12.*PI*PI);
	OM1 = ((-1. + Nc*Nc)*(-2.*pow(mu,4.) + 3.*pow((ln*m*m - mu*u),2.) + (4. + 6.*log((lambda)/m))*m*m*(-(ln*m*m) + mu*u))*alp)/(16.*PI*PI*PI);
	return OM0+OM1;
}
/* De afgeleide naar mu */
double dthpotmas(double mu)
{
	double dOM0,dOM1;
	dOM0 = -(Nc*(3.*pow(m,3.)*(m*dln + 4.*ln*dm) + u*(6.*mu*mu - 5.*m*(m + 2.*mu*dm)) + mu*(2.*mu*mu - 5.*m*m)*du))/(24.*PI*PI);
	dOM1 = ((-1. + Nc*Nc)*(alp*(-8.*pow(mu,3.) + 2.*(4. + 6.*log((lambda)/m))*m*(-ln*m*m + mu*u)*dm - (6.*m*(ln*m*m - mu*u) * (m - mu*dm))/mu + 6.*(ln*m*m - mu*u)*(-u + m*m*dln + 2.*ln*m*dm - u*du) + (4. + 6.*log((lambda)/m))*m*m*(u - m*(m*dln + 2.*ln*dm) + mu*du)) + (-2.*pow(mu,4.) + ((-4. + 3.*ln - 6.*log((lambda)/m))*m*m - 3.*mu*u)*(ln*m*m - mu*u))*dalp))/(16.*pow(PI,3.));
 	return dOM0 + dOM1;
}
	
/* Tweede afgeleide naar mu */
double ddthpotmas(double mu)
{
	double ddOM0,ddOM1;
	ddOM0 = -(Nc*(12.*pow(m,3.)*dln*dm + 18.*ln*pow(m,2.)*pow(dm,2.) + 2.*u*(2.*mu - 5.*m*dm) + 2.*(pow(mu,2.) - (5.*pow(m,2.))/2.)*du + 2.*mu*(2.*mu - 5.*m*dm)*du + (3.*pow(m,4.)*ddln)/2. + 6.*ln*pow(m,3.)*ddm + mu*u*(2. - 5.*pow(dm,2.) - 5.*m*ddm) + mu*(pow(mu,2.) - (5.*pow(m,2.))/2.)*ddu))/(12.*pow(PI,2.));
	ddOM1 = ((-1. + pow(Nc,2.))*(-8.*pow(mu,3.) + 2.*(4. + 6.*log((lambda)/m))*m*(-(ln*pow(m,2.)) + mu*u)*dm + (6.*pow(m,3.)*(-(ln*pow(m,2.)) + mu*u)*(lschaal/m - (lambda*dm)/pow(m,2.)))/(lambda) + 6.*(ln*pow(m,2.) - mu*u)*(-u + pow(m,2.)*dln + 2.*ln*m*dm - mu*du) + (4. + 6.*log((lambda)/m))*pow(m,2.)*(u - pow(m,2.)*dln - 2.*ln*m*dm + mu*du))*dalp)/(8.*pow(PI,3.)) + ((-1. + pow(Nc,2.))*alp*(-24.*pow(mu,2.) + 2.*(4. + 6.*log((lambda)/m))*(-(ln*pow(m,2.)) + mu*u)*pow(dm,2.) - (6.*pow(m,3.)*(-(ln*pow(m,2.)) + mu*u)*(lschaal/m - (lambda*dm)/pow(m,2.)))/(lschaal*pow(mu,2.)) + (30.*pow(m,2.)*(-(ln*pow(m,2.)) + mu*u)*dm*(lschaal/m - (lambda*dm)/pow(m,2.)))/(lambda) + 6.*pow(-u + pow(m,2.)*dln + 2.*ln*m*dm - mu*du,2.) + 4.*(4. + 6.*log((lambda)/m))*m*dm*(u - pow(m,2.)*dln - 2.*ln*m*dm + mu*du) + (12.*pow(m,3.)*(lschaal/m - (lambda*dm)/pow(m,2.))*(u - pow(m,2.)*dln - 2.*ln*m*dm + mu*du))/(lambda) + 2.*(4. + 6.*log((lambda)/m))*m*(-(ln*pow(m,2.)) + mu*u)*ddm + (6.*pow(m,3.)*(-(ln*pow(m,2.)) + mu*u)*((-2.*lschaal*dm)/pow(m,2.) + (2.*lambda*pow(dm,2.))/pow(m,3.) - (lambda*ddm)/pow(m,2.)))/(lambda) + 6.*(ln*pow(m,2.) - mu*u)*(4.*m*dln*dm + 2.*ln*pow(dm,2.) - 2.*du + pow(m,2.)*ddln + 2.*ln*m*ddm - mu*ddu) + (4. + 6.*log((lambda)/m))*pow(m,2.)*(-4.*m*dln*dm - 2.*ln*pow(dm,2.) + 2.*du - pow(m,2.)*ddln - 2.*ln*m*ddm + mu*ddu)))/(16.*pow(PI,3.)) + ((-1. + pow(Nc,2.))*(-2.*pow(mu,4.) + 3.*pow(ln*pow(m,2.) - mu*u,2.) + (4. + 6.*log((lambda)/m))*pow(m,2.)*(-(ln*pow(m,2.)) + mu*u))*ddalp)/(16.*pow(PI,3.));
	return ddOM0 + ddOM1;
}
	
/* Thermische potentiaal zonder massa */
double thpotnomas(double mu)
{
	double antw,zero,first;
	zero = 1.;
	first = -2.0*(alp/PI);
	antw = -(pow(mu,4.0))/(4.0*PI*PI)*(zero+first);
	return antw;
}

/* De afgeleide */
double dthpotnomas(double mu)
{
	double antw;
	antw = (pow(mu,3.)*(-2.*PI + 4.*alp + mu*dalp))/(2.*pow(PI,3.));
	return antw;
}
/* En de tweede afgeleide */
double ddthpotnomas(double mu)
{
	double antw;
	antw = (pow(mu,2.)*Nf*(-6.*PI + 12.*alp + 8.*mu*dalp + pow(mu,2.)*ddalp))/(2.*pow(PI,3.));
	return antw;
}


/* Thermische potentiaal vrij gas (elektrongas)*/
double thpotfree(double mu)
{
	double antw;
	antw = -(pow(mu,4.))/(4.*PI*PI);
	return antw;
}

/* De eerste afgeleide */
double dthpotfree(double mu)
{
	double antw;
	antw = -(pow(mu,3.))/(PI*PI);
	return antw;
}
/* De tweede afgeleide */
double ddthpotfree(double mu)
{
	double antw;
	antw = (-3.*pow(mu,2.))/(PI*PI);
	return antw;
}

double xfunc (double x)
{
	double antw=0.;
	lschaal=(scaling/(3.))*(2.+ x);
	defin(vars.mu);
	vars.nd = -dthpotnomas(vars.mu)*toNAT;
	if(!finite(vars.nd)) vars.nd = 0.;
	vars.ns = -dthpotmas(vars.mu)*toNAT;
	if(!finite(vars.ns)) vars.ns = 0.;
	vars.nu = -dthpotnomas(vars.mu*x)*toNAT;
	if(!finite(vars.nu)) vars.nu = 0.;
	vars.ne = - dthpotfree(vars.mu*(1.-x))*toNAT;
	if(!finite(vars.ne)) vars.ne = 0.;
	antw = (2./3.)*vars.nu - (1./3.)*vars.nd - (1./3.)*vars.ns - vars.ne;
	return antw;
}

void calc (double mu)
{
	double xlow=1.,xmid,xhigh=1.;
	double flow,fhigh,fmid,test;
	double xacc=1e-6;
	double schaalacc=1e-3;
	double depsdn; /* Gibbs-potetniaal: \mu_{u} + \mu_{d} + \mu_{s} = (2.+x)*mu*/
	double thpot,munr,lschaalsave=0.;
	vars.mu = mu;
/*	if(!finite(-dthpotmas(vars.mu))) {
		setnull();
		return;
	}*/
	/* Nulpunt isoleren */
	do {
		xlow = -0.1;
		flow = xfunc(xlow);
	} while(flow > 0. && xlow > 0.);
	/* Root finding om vgl op te lossen: Som(n(i,mu)*Q=0, daarmee mu(u)==x en mu(e)==1-x vinden)*/
	vars.x = (*zbrent)(xfunc,xlow,xhigh,xacc);

	vars.lschaal = lschaal;
	/*if(!finite(vars.ns)||vars.x<0.||vars.x>1.||vars.nu<0.||vars.nd<0.||vars.ne<0.||vars.ns<0.){
		printf("# Bij mu=%g :\tx=%g\tns=%g\n",vars.mu,vars.x,vars.ns);
		setnull();
		return;
	}*/
	vars.ths = thpotmas(vars.mu)*toNAT;
	vars.thd = thpotnomas(vars.mu)*toNAT;
	vars.thu = thpotnomas(vars.x*vars.mu)*toNAT;
	vars.the = thpotfree((1.-vars.x)*vars.mu)*toNAT;
	vars.thf = thpotfree(vars.mu)*toNAT;
	vars.thtot = vars.ths + vars.thd + vars.thu + vars.the;
	munr =  vars.mu*vars.nd + vars.mu*vars.ns + vars.mu*vars.x*vars.nu + (1.-vars.x)*vars.mu*vars.ne;
	//printf("Bij mu = %g :\tmu*nd = %g\tmu*ns=%g\tmu*nu=%g\tmu*ne=%g\tthtot=%g\n",vars.mu,vars.mu*vars.nd/vars.thf,vars.mu*vars.ns/vars.thf,vars.mu*vars.x*vars.nu/vars.thf,(1.-vars.x)*vars.mu*vars.ne/vars.thf,vars.thtot/vars.thf);
	vars.eps = (vars.thtot + munr); 
	vars.nb =  (1./3.)*(vars.nu + vars.nd + vars.ns);
	depsdn = (2.+vars.x)*vars.mu;
	vars.p = (vars.nb*depsdn + vars.ne*vars.mu*(1.-vars.x)) - vars.eps;
	//printf("Mu: %g\tx: %g\tlschaal: %g\n",vars.mu,vars.x,lschaal);
	if(!finite(vars.p)) setnull();
	return;
}

void setnull()
{
	vars.x=0.;
	vars.nu=0.;
	vars.nd=0.;
	vars.ns=0.;
	vars.ne=0.;
	vars.nb=0.;
	vars.eps=0.;
	vars.p=0.;
	vars.thu=0.;
	vars.thd=0.;
	vars.ths=0.;
	vars.the=0.;
	vars.thf=0.;
	vars.thtot=0.;
}
