/* This program solves a set of coupled differtial equations descibing the structure of a NS: dp/dr=(G*M/r^2)*rho and dM/dr=4*PI*r^2*rho. The differential equations are represented by the array dydx[n] where n is the number of different equations. Here y[0] is the chemical potential and y[1] is the mass. The intial conditions are r=0, M(0)=0 and p(0)=po.The procedure stops when the pressure has reached a certain value pstop. The method of solving the equations is a fifth order Runge Kutta method (standard for this kind of problems), using Kash-Carp parameters. See "Numerical Recipes for C" chapter 16 for more details on this method. A simple variable stepsize is used according to Baym, Petchick and Sutherland 1971.*/
/* Programma is aangepast voor het oplossing van de structuur van een quarkster. Dit gebreurt voor een massareeks, waarna de massa's worden weggeschreven zodat een massa-straaldiagram kan worden gemaakt.
Voor het oplossen van mu uit de druk wordt gebruik gemaakt van een simpel numeriek oplosschema uit Nemerical Recipes voor C. De toenstantvergelijkingen zijn verkregen met behulp van eerste orde QCD-storingsrekening, moor het MS-barschema (hoewl eerste orde schema-onafhankelijk is). De labda_MS-schaal is bepaald volgens de methode van Fraga et al (hep-ph/0101143v1). Hiermee is alleen labda nog vrij.*/

#include <stdio.h>
#include <math.h>
#include <string.h>

#define NPOINTS 99
#include "qcdeqn_mas.h"
#include "natunits.h"
#include "settings.h"
#include "nrutil.h"
#include "settings.h"

#define stdhe 197.32 /* hbar c naar MeV fm^-1 */
#define fm 1e-15 /* Een femtometer */
#define INT_METHOD "bsstep"
#include "bsstep.h"

const double toSI=mev/(fm*fm*fm);
const double toNAT=1./(stdhe*stdhe*stdhe);

double tov(double m, double mu, double r);
double cont(double r);
void differential(double, double [], double []);
double lowpres (double mu);

extern stor vars;

/* Het hoofdprogramma */
int main (argc, argv)
int argc; 
char *argv[];
{
int nmustep=NPOINTS,imu;
int i,k,kmax=200000;
int nok=0,nbad=0;
double mu0,p0,eps0,pmax=1e3,mustep,mustart,mumax,r0=1e-12, m0=1e-13,pstop=0.,m=0.,r=0.,h,htry=1.0e-6,hnext,hdid,hmin=1e-20;
double *y,*dydx,*yscale;
double mu,x;
double tolerance = 1e-4	;
char qout[80],qmassas[80],dir[80];

y=dvector(1,n);
dydx=dvector(1,n);
yscale=dvector(1,n);

/*c1,...,c6 are KC parameters for 4th order and c11,...,c61 for 5th order. The dc numbers indicate the difference between the 4th and 5th order method and are used to estimate the error of the method.*/
 
/*Open file to store data in. The results are written to an asci file*/
strcpy(dir,"/net/astro008/scratch/srhardem/afstudeer/output/");
strcpy(qout,dir);
strcpy(qmassas,dir);
if(argc==1) {
	strcat(qout,"qout.asc");
	strcat(qmassas,"qmassas.asc");
}
if(argc==2) {
	strcat(qout,argv[1]);
	strcat(qmassas,argv[1]);
	strcat(qout,"_out.asc");
	strcat(qmassas,"_massas.asc");
}
	if(argc==3) {	printf("%g\t%g\n",vars.p,vars.eps);
	strcat(qout,argv[1]);
	strcat(qmassas,argv[2]);
	strcat(qout,".asc");
	strcat(qmassas,".asc");
}

printf("Opent files %s en %s voor massa-straalbestand en logbestand\n",qmassas,qout);
FILE *fp;
FILE *totmas;
  fp=fopen(qout,"w");
  totmas=fopen(qmassas,"w");

/* Bereken p(mu)=0 */
//mustop = zbrent(lowpres,200.,500.,1e-5);
mumax=650.;
mustart=381.;
mustep=log10(mumax/mustart)/nmustep;

printf("## mustart = %g\tmumax =  %g\n",mustart,mumax);


/* Header in massauitvoerfile*/
fprintf(totmas,"# Massa's berekend met hydromass.c\n");
fprintf(totmas,"# Parameters: mustart=%15.6g\tmumax=%15.6g\tnmustep=%i\tmustep=%15.6g\n",mustart,mumax,nmustep+1,mustep);
fprintf(totmas,"# \n#\n# Steps\t\tr[km]\tM[Msun]\t mu_0(MeV)\tEps_0(MeV/fm^-3]\tP_0[MeV/fm^-3]\n");

for (imu=0;imu<nmustep+1;imu++)
	{
	mu0=mustart*pow(10,imu*mustep);
/*first stepsize to try*/
h=htry;

/*initial conditions*/  
x=r0;
y[1]=mu0;
k=0;
y[2]=m0;
calc(mu0);
eps0=vars.eps;
p0=vars.p;
/*Printing table header*/ 
fprintf(fp,"# \n\n Centrale chemische potentiaal: %15.6g\n",mu0);
		fprintf(fp,"# Step\tr[km]\tM[Mzon]\tmu[MeV]\teps[MeV/fm^3]\tp[MeV/fm^3]\tStepsize\n");

printf("Model bij chemische potentiaal %15.6g\n",mu0);
while (k<=kmax && vars.p>pstop && h>0. &&m>=0.)
{

	/*dummy variables*/
	mu=y[1];
	m=y[2]/Msun;
	r=x/1000.0;

	yscale[1] = 1.0;
	yscale[2] = Msun;
	(*bsstep)(y,dydx,n,&x,h,tolerance,yscale,&hdid,&hnext,differential);
	if (hdid == h) ++nok; else ++nbad;
	if (fabs(hnext) <= hmin) nrerror("Stepsize problem");
	h=hnext;
	/*Increase k to start the new integration loop*/
	k++;
	if(k>=kmax) nrerror("kmax bereikt --> te veel stappen!");

	/*Printing the parametervalues after this loop*/
	if(imu%1==0){
		/* bespaar ruimte */
		fprintf(fp,"%i\t%g\t%g\t%g\t%g\t%g\t%g\n",k,r,m,mu,vars.eps,vars.p,h);
	}


	/*End of while loop*/
}


fprintf(totmas,"%i\t%g\t%g\t%g\t%g\t%g\n",k,r,m,mu0,eps0,p0);
fprintf(fp,"#---> nok = %i, nbad = %i\n",nok,nbad);
k=0;
r=0;
mu=0;
m=0;
h=0;
nok=0;
nbad=0;
/*End of loop*/
}


fclose(fp);
fclose(totmas);

free_dvector(y,1,n);
free_dvector(dydx,1,n);

return 0;
}


/* TOV-vergelijking */
double tov(double m, double mu, double r)
{
	double p,rho,dmudr,dmudr1,dpdr,dpdmu;
	/* Convert to SI units */
	p=vars.p*toSI/(c*c);
	rho=vars.eps*toSI/(c*c);
	dpdr=-(G)/(r*r)*(rho+p)*(m+4.0*PI*pow(r,3.0)*p)/(1.0-2.0*G*m/(r*c*c));
	/* Delen door toSI omdat dpdr = J/m^3/m moet naar MeV/fm^3/m  */
	dmudr1 =  dpdr/toSI/(3.*vars.nb+vars.ne);
	dpdmu = (-3.*dthpotmas(vars.mu) - 3.*dthpotnomas(vars.mu) - (-1. + vars.x)*(dthpotmas(vars.mu) + dthpotnomas(vars.mu) - 2.*dthpotnomas(vars.x*vars.mu)) - 3.*vars.x*dthpotnomas(vars.x*vars.mu) - (-1. + vars.x)*vars.mu*(ddthpotmas(vars.mu) + ddthpotnomas(vars.mu) - 2.*vars.x*ddthpotnomas(vars.x*vars.mu)));
	dmudr = (2.+vars.x)*(dpdr/toSI)/(dpdmu*toNAT);
	//printf("%g\t%g\t%g\n",dmudr,dmudr1,dmudr1/dmudr);
	return dmudr;
}

/* Continuiteitsvergelijking */
double cont(double r)
{
	double rho;
	rho = vars.eps*toSI/(c*c);
	return 4.0*PI*r*r*rho;
}

/*function representing the differential equations*/
void differential(double x, double y[], double dydx[])
{
	/* y[1] is mu, y[2] is m */
	defin(y[1]);
	calc(y[1]);
	dydx[1]=tov(y[2],y[1],x);
	dydx[2]=cont(x);
	return;
}

double lowpres (double mu)
{
	calc(mu);
	return vars.p;
}
